#!/bin/bash

if [ $# != 1 ] ; then
echo "Please use the F3_env_setup.sh:"
echo " "
echo "Rtl: source ./xbinst_oem/F3_env_setup.sh xdma"
echo " "
echo "SDAccel: source ./xbinst_oem/F3_env_setup.sh xocl"
echo " "
return 0
fi

yum -q -y install python-devel >/dev/null 2>&1
yum -q -y install pciutils >/dev/null 2>&1
yum -q -y install kernel-devel >/dev/null 2>&1
yum -q -y install unzip >/dev/null 2>&1
pip -q install oss2
pip -q install --upgrade aliyun-python-sdk-faas 
pip -q install aliyun-python-sdk-ram  

#======================== get working directory ==========================================================================

SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
  TARGET="$(readlink "$SOURCE")"
  if [[ $TARGET == /* ]]; then
    SOURCE="$TARGET"
  else
    DIR="$( dirname "$SOURCE" )"
    SOURCE="$DIR/$TARGET" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
  fi
done
DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

#====================================== check and update faascmd version ===========================================================

faascmd_loc=$DIR/runtime/bin/faascmd
last_faascmd_etag=`curl -s -I http://fpga-tools.oss-cn-shanghai.aliyuncs.com/faascmd | grep "ETag: " | cut -d "\"" -f 2`
if [ $? != 0 ]; then
    echo -e "\033[31m the faascmd is not found in faas tool's oss \033[0m"
fi

if [[ -f $DIR/.faascmd_etag ]] && [[ -f $DIR/runtime/bin/faascmd ]]; then 
    now_faascmd_etag=`cat $DIR/.faascmd_etag`
else
    now_faascmd_etag=''
fi
if [ "$last_faascmd_etag" != "$now_faascmd_etag" ];
then
    if [ -f $DIR/runtime/bin/faascmd ]; then
        \rm $DIR/runtime/bin/faascmd
    fi
    wget -q http://fpga-tools.oss-cn-shanghai.aliyuncs.com/faascmd -P $DIR/runtime/bin
    chmod 555 $DIR/runtime/bin/faascmd
    echo "$last_faascmd_etag" >  $DIR/.faascmd_etag
    echo -e "The\033[32m faascmd\033[0m  updated successfully"
fi

source /opt/Xilinx/SDx/2018.2/settings64.sh

if [ $1 = "xdma" ]; then
	f3_driver="xdma"
	installscript="install.sh"
	setupscript="setup.sh"
elif [ $1 = "xocl" ]; then 	
	f3_driver="xocl"
	installscript="install_ocl.sh"
	setupscript="setup_ocl.sh" 
else
	echo "Please enter a valid driver option:"
	echo "for RTL project: xdma; for SDAccel project: sda"
fi

driver_info=`lsmod | grep $f3_driver`
if [ "x$driver_info" == "x" ] ;then
	cd $DIR
	echo $installscript
	sh ./"$installscript"
	cd -
	driver_info=`lsmod | grep $f3_driver`
fi

if [ -z $XILINX_OPENCL ] || [ $XILINX_SDX ];then
	cd $DIR
        #source ./setup.sh
	source ./"$setupscript"
	cd - >/dev/null 2>&1
fi


echo "----------------------------------------------------"

echo -n "Checking whether "$f3_driver" driver exists : "
if [ "x$driver_info" != "x" ] ;then
	echo -e "\e[0;32m exist \e[0m"
else
	echo -e "\e[0;33m did not exist \e[0m"
fi

echo "----------------------------------------------------"

vivado_info=`which vivado 2> /dev/null`
echo -n "Checking whether vivado exists : "
if [ "x$vivado_info" == "x" ] ; then
    vivado_info=`cat $WORK_DIR/.find_tmp | grep vivado | grep $vivado_ver_req | grep -v unwrapped | sort -r`
    echo -e "\e[0;33m did not exist \e[0m"
else
    echo -e "\e[0;32m $vivado_info \e[0m"
fi

echo "----------------------------------------------------"
echo -n "Checking whether vivado license has been set : "
export LM_LICENSE_FILE=2100@faas-cert-sh.alibaba-inc.com
#export LM_LICENSE_FILE=2100@faas-cert-hz1.alibaba-inc.com

if [ -z $LM_LICENSE_FILE ];then
        echo -e "\e[0;33m not exists \e[0m"
else
        echo -e "\e[0;32m $LM_LICENSE_FILE  \e[0m"
fi

echo "----------------------------------------------------"
echo -n "Checking whether vivado aliyun FaaS platform exists : "

platform_info=`xocc --list_xdevices | grep xilinx_aliyun-f3_dynamic_5_0`
if [ "x$platform_info" == "x" ] ; then
	echo -e "\e[0;33m did not exist \e[0m"
else
	echo -e "\e[0;32m $platform_info \e[0m"
fi

echo "----------------------------------------------------"

echo -n "Checking whether runtime library exists : "
if [  "x$XILINX_OPENCL" != "x" ] ; then
	echo -e "\e[0;32m exist \e[0m"
else
	echo -e "\e[0;33m did not exist \e[0m"
fi
echo "----------------------------------------------------"

export PATH=$DIR/runtime/bin:$PATH

faascmd_info=`which faascmd 2> /dev/null`
echo -n "Checking whether faascmd exists : "
if [  "x$faascmd_info" != "x" ] ; then
	echo -e "\e[0;32m exist \e[0m"
else
        echo -e "\e[0;33m did not exist \e[0m"
fi
echo "----------------------------------------------------"

