#!/bin/bash
#script脚本名
script=${BASH_SOURCE[0]}
#脚本绝对路径名
full_script=$(readlink -f $script)
#脚本名
script_name=$(basename $full_script)
#脚本目录名
script_dir=$(dirname $full_script)

#echo $script
#echo $full_script
#echo $script_name
#echo $script_dir

debug=0

function info_msg {
  echo -e "Aliyun FaaS FPGA-SDK-INFO: $1"
}

function err_msg {
  echo -e >&2 "Aliyun FaaS FPGA-SDK-ERROR: $1"
}

function usage {
  echo -e "USAGE: $script_name [-h|-help] [-xclbin=<xclbin_filename>] [-o=<aliyunxclbin_filename>]"
}
#FIXME -o

function help {
  info_msg "$script_name"
  info_msg " "
  info_msg "SDAccel Packdge"
  info_msg " "
  info_msg "sdaccle_package.sh assumes you have:"
  info_msg "  (*) Read the README on Github and understand the SDAccel workflow (SDAccel/README.md)"
  info_msg "  (*) Generated an XCLBIN using the SDAccel Build flow"
  info_msg "  (*) Ready to create an Aliyun FPGA image and test on F2.  Your kernel has been validated using SW/HW Emulation."
  info_msg "sdaccle_package.sh will:"
  info_msg "  (1) Split your XCLBIN into two parts:  DCP (.bit) and Metadata (.xml)"
  info_msg "  (2) Generates a Manifest file for AFI creation that sets the clocks based on your achieved target freq from your build"
  info_msg "  (3) Prepares tar file for AFI creation process"
  echo " "
  usage
}



if [ "$1" == "" ]
then
  echo "ERROR: invalid usage"
  usage
  exit 1
fi

while [ "$1" != "" ]; do
    PARAM=`echo $1 | awk -F= '{print $1}'`
    VALUE=`echo $1 | awk -F= '{print $2}'`
    case $PARAM in
        -h | --help)
            usage
            exit
            ;;
        -xclbin)
            xclbin=$VALUE
            ;;
        -o)
            aliyunxclbin=$VALUE
            ;;
        *)
            echo "ERROR: unknown parameter \"$PARAM\""
            usage
            exit 1
            ;;
    esac
    shift
done


if [[ -e "$xclbin" ]]
then
    echo "Found xclbin '$xclbin'"
else
    echo "Error: File '$xclbin' is not found"
    exit 1
fi

stripped_xclbin=$(basename $xclbin)
ext_xclbin=${stripped_xclbin##*.}
stripped_xclbin=${stripped_xclbin%.*}
#echo $stripped_xclbin

if [ "$aliyunxclbin" == "" ]
then
    aliyunxclbin=$stripped_xclbin
fi


if [ "$aliyunxclbin" != "$stripped_xclbin" ]
then
    echo "WARNING!!!"
    echo "Warning: $aliyunxclbin does not match $stripped_xclbin"
    echo "Warning: For github examples, -o <xclbin_filename> must be equal to $stripped_xclbin"
    echo "WARNING!!!"
fi

if [[ -e "$aliyunxclbin" ]]
then
    echo "Error: File '$aliyunxclbin' already exists"
    exit 1
fi



timestamp=$(date +"%y_%m_%d-%H%M%S")

#Steps
#1. Strip XCLBIN to get DCP for ingestion
#2. Create Manifest file
#3. Prepare ingestion tar file


#STEP 1
#Strip XCLBIN to get DCP for ingestion
$XILINX_SDX/runtime/bin/xclbinsplit -o ${timestamp} $xclbin
if [[ -e "${timestamp}-primary.bit" ]]
then
    echo "Split DCP from xclbin: ${timestamp}-primary.bit"
else
    echo "Error: File ${timestamp}-primary.bit is not found"
    exit 1
fi
if [[ -e "${timestamp}-xclbin.xml" ]]
then
    echo "Split Metadata from xclbin: ${timestamp}-primary.bit"
else
    echo "Error: File ${timestamp}-xclbin.xml is not found"
    exit 1
fi

if [[ -d "to_aliyun" ]]
then
    echo "Error: Directory to_aliyun already exists"
    exit 1
fi

mkdir to_aliyun
cp ${timestamp}-primary.bit to_aliyun/${timestamp}_SH_CL_routed.dcp
cp ${timestamp}-xclbin.xml to_aliyun/${timestamp}-xclbin.xml


#STEP 3
#Prepare ingestion
tar -zcvf ${timestamp}_SDAccel_Kernel.tar.gz to_aliyun/${timestamp}_SH_CL_routed.dcp to_aliyun/${timestamp}-xclbin.xml

#
