#!/bin/sh

# check if input file name
if [ $# -lt 1 ]; then
  echo "Please specify the image name" 
  echo "use faascmd list_images to find Name"
  echo "sh ./xbinst_oem/tool/faas_download_image.sh [image name] [FPGA_ID 0-3]" 
  exit 1
fi

# check if input fpga id
if [ $# -lt 2 ]; then
  echo "Please specify the FPGA ID for downloading the image" 
  exit 1
fi

# check fpga status
check_status()
{
FpgaStatus="$(jq --arg index ${i} '[.Instances.instance[].FpgaStatus]|.[$index|tonumber]' .listinstanceoutput.txt)"
echo "FPGA "${i}" current status: "${FpgaStatus}
}

YourInstanceId=`curl -s http://100.100.100.200/latest/meta-data/instance-id`
faascmd list_instances --instanceId=${YourInstanceId} > .listinstanceoutput.txt
cat .listinstanceoutput.txt | grep Error
FPGA_num="$(jq -r '[.Instances.instance[].FpgaUUID]| length' .listinstanceoutput.txt)"
echo "There are "${FPGA_num}" FPGAs on this instance"

# check if image is valid
faascmd list_images > .listimagesoutput.txt
cat .listimagesoutput.txt | grep Error
Myobject_tmp=$1
Myobject=${Myobject_tmp##*/}
FPGAImageUUID="$(jq -r --arg name ${Myobject} '.FpgaImages.fpgaImage[] | select(.Name == $name) | .FpgaImageUUID' .listimagesoutput.txt)"
echo "FPGA Image ID is "${FPGAImageUUID}


FpgaImageStatesuccess=$(jq --arg FPGAImageUUID ${FPGAImageUUID} '.FpgaImages.fpgaImage[] | select(.FpgaImageUUID == $FPGAImageUUID) | .State' .listimagesoutput.txt)
if [ "${FpgaImageStatesuccess}" != "\"success\"" ]; then
        echo "fpga image is not ready"
else
        # check if user fpga (0-3) is valid
        for i in ${@:2}; do
                if [[ ${i} > $((${FPGA_num} + 1)) ]]; then
                        echo "ERROR: no such id on this instance"
                fi
                #check fpga status
                check_status ${i}
        done
fi

# download image to each target fpga 
for i in ${@:2}; do
	FpgaUUID="$(jq -r --arg index ${i} '[.Instances.instance[].FpgaUUID]|.[$index|tonumber]' .listinstanceoutput.txt)"
	echo "Start programming FPGA "${i}" id:"${FpgaUUID}
	faascmd download_image  --instanceId=${YourInstanceId} --fpgauuid=${FpgaUUID} --fpgatype=xilinx --imageuuid=${FPGAImageUUID} --imagetype=afu --shell=f30001 > .downloadimageoutput$i.txt
	cat .downloadimageoutput$i.txt | grep Error
	faascmd fpga_status --fpgauuid=${FpgaUUID} --instanceId=${YourInstanceId} > .downloadstatusoutput$i.txt
	cat .downloadstatusoutput$i.txt | grep Error
	downloadstatus=$(jq '.TaskStatus' .downloadstatusoutput$i.txt)
	while [ "${downloadstatus}" != "\"valid\"" ]
	do
		sleep 10	
		faascmd fpga_status --fpgauuid=${FpgaUUID} --instanceId=${YourInstanceId} > .downloadstatusoutput$i.txt
		downloadstatus=$(jq '.TaskStatus' .downloadstatusoutput$i.txt)
		echo "FPGA "${i}" download status: "${downloadstatus}
	done
	echo "fpga image has been download to FPGA "${i}" fpgauuid: "${FpgaUUID}
done

rm -f .downloadstatusoutput.txt
rm -f .listimagesoutput.txt
rm -f .listinstanceoutput.txt
rm -f .downloadimageoutput.txt
