#!/bin/sh

echo "shell verison f30010"

echo " "
if [ $# -lt 1 ]; then
  echo "Please submit xxx.tar.gz " 
  echo "sh ./xbinst_oem/tool/faas_upload_and_create_image.sh xxx.tar.gz" 
  exit 1
fi

# install jq to parse json
yum install jq -y -q >/dev/null 2>&1

Myobject_tmp=$1
Myobject=${Myobject_tmp##*/}
echo "uploading: "${Myobject}

# get instance id
YourInstanceId=`curl -s http://100.100.100.200/latest/meta-data/instance-id`
echo "Instance Id: "${YourInstanceId}

# check if filename already exist in OSS
faascmd list_images > .listimagesoutput.txt
cat .listimagesoutput.txt | grep Error
echo $Myobject
nameexist=$(jq --arg objname ${Myobject} '.FpgaImages.fpgaImage[] | select(.Name == $objname) | .Name' .listimagesoutput.txt)
if [[ $nameexist != '' ]]; then 
	echo "ERROR: File name already exists in OSS! please use another name instead"
	exit 2
fi

# upload
faascmd upload_object --object=${Myobject} --file=${Myobject_tmp}
faascmd create_image --object=${Myobject} --fpgatype=xilinx --name=${Myobject} --tags=hereIsFPGAImageTag --encrypted=false --shell=f30010 > .createimageoutput.txt

# try capture error during creat image 
cat .createimageoutput.txt | grep Error

# get FPGAImageUUID and break if error
FPGAImageUUID=$(jq -r '.FpgaImageUUID' .createimageoutput.txt)
if [ $FPGAImageUUID == '' ]; then
	echo "ERROR: Failed to get FPGAImageUUID!"
	exit 2
else
	echo "image uuid: "${FPGAImageUUID}
fi

# try capture error during list_images
faascmd list_images > .listimagesoutput.txt
cat .listimagesoutput.txt | grep Error

# get FpgaImageStatesuccess and break if error
FpgaImageStatesuccess=$(jq --arg FPGAImageUUID ${FPGAImageUUID} '.FpgaImages.fpgaImage[] | select(.FpgaImageUUID == $FPGAImageUUID) | .State' .listimagesoutput.txt)
if [ $FpgaImageStatesuccess == '' ]; then
	echo "ERROR: Failed to get FpgaImageStatesuccess!"
	exit 2
else
	echo "image state: "${FpgaImageStatesuccess}
fi
# wait till creating image succeed
while [ "${FpgaImageStatesuccess}" != "\"success\"" ]
do	
	sleep 60
	faascmd list_images > .listimagesoutput.txt		 
	FpgaImageStatesuccess=$(jq --arg FPGAImageUUID ${FPGAImageUUID} '.FpgaImages.fpgaImage[] | select(.FpgaImageUUID == $FPGAImageUUID) | .State' .listimagesoutput.txt)
	echo "image state: "${FpgaImageStatesuccess}
	if [ "${FpgaImageStatesuccess}" == "\"failed\"" ]; then
		echo "ERROR: Online compiling failed. Please check faas_compiling.log for details."
		BASEDIR=$(dirname "$0")
		sh $BASEDIR/faas_checklog.sh ${Myobject}
		exit 2
	fi
done
echo "FPGA image compilation succeed, start check FPGA status"


# try capture error during list_instances
faascmd list_instances --instanceId=${YourInstanceId} > .listinstanceoutput.txt
cat .listinstanceoutput.txt | grep Error

# get FPGA number and break if error
FpgaNum=$(jq '[.Instances.instance[].FpgaUUID]| length' .listinstanceoutput.txt)
if [ $FpgaNum == '' ]; then
	echo "ERROR: Failed to get FpgaNum!"
	exit 2
else
	echo "Total number of FPGAs on this instance: "${FpgaNum}
fi
# check FPGA status
for number in `seq 0 $((${FpgaNum} - 1))`
do
        FpgaUUID="$(jq -r --arg index ${number} '[.Instances.instance[].FpgaUUID]|.[$index|tonumber]' .listinstanceoutput.txt)"
        FpgaStatus="$(jq --arg index ${number} '[.Instances.instance[].FpgaStatus]|.[$index|tonumber]' .listinstanceoutput.txt)"
        while [ "${FpgaStatus}" != "\"valid\"" ]
        do
                sleep 10
                FpgaStatus="$(jq '.Instances.instance[].FpgaStatus' .listinstanceoutput.txt)"
                echo "FPGA "${number}" current status: "${FpgaStatus}
        done
done

echo "FPGA is ready to load image"

rm -f .listimagesoutput.txt
rm -f .listinstanceoutput.txt
rm -f .createimageoutput.txt

